function SwipeController($scope, ScreenService, templateManager, $rootScope, $timeout, $document) {

	var resizeSwipes = function() {
		var container = $(".zh-swipe-container");
		var swipes = container.children();
		for (var i = 0; i < $rootScope.swipes.length; i++) {
			var swipe = $(swipes[i]);
			if (i === 0) {
				// first swipe always have 100% of container
				swipe.css('width', '100%');
			} else {
				swipe.css('width', (i === $rootScope.swipes.length - 1 ? 75 - ($rootScope.swipes.length - 2) * 10 : 100 - (i * 10)) + '%');
			}
		}
		$timeout(function() {
			for (var i = 0; i < $rootScope.swipes.length; i++) {
				$rootScope.swipes[i].open = true;
			}
			$(window).trigger('resize.grid', container);
		});
	};

	$rootScope.swipes = [];
	$scope.$on("openSwipe", function(e, widget, closeAll, deferred, customClass) {
		//TODO: Check if this is the correct behaviour: the closeSwipe function makes more than just removing from the array
		if (closeAll || !$rootScope.swipes) {
			$rootScope.swipes = [];
		}

		// $scope.widget = widget
		// if widget is not inside $rootScope.swipes, a new swipe is added
		if ($rootScope.swipes.indexOf(widget) === -1) {
			// on master detail grid when open detail swipe (mode == view) parent swipe is the actual widget
			if (widget.template.indexOf('master_detail/grid.html') !== -1 && widget.mode == ViewMode.VIEW) {
				widget.swipeParent = widget;
			} else {
				widget.swipeParent = widget.getParent();
			}
			widget.lastActiveWidget = widget.swipeParent;

			$rootScope.swipes.push(widget);

			var element = $(".zh-swipe-container");

			if (customClass) {
				element.addClass(customClass);
			}

			if (!element.hasClass("open")) {
				element.addClass("open");
			}
			$timeout(resizeSwipes);
		} else {
			// if widget is inside $rootScope.swipes then closes all swipes beyond current level
			var currentLevel = $rootScope.swipes.indexOf(widget) + 1;
			ScreenService.closeSwipe(currentLevel);
		}

		// delay the broadcast so the swipe can be build by ngRepeat
		$timeout(function() {
			// this function is used by list and grid to positioning the tip
			$rootScope.$broadcast("swipeOpened");
			deferred.resolve();
		}, 300);
	});

	$scope.$on("closeSwipe", function(e, level, deferred) {
		var element = $(".zh-swipe-container");

		if ($rootScope.swipes && $rootScope.swipes.length > 0) {
			// if level is not defined than only the last swipe is closed
			if (Util.isUndefined(level)) {
				level = $rootScope.swipes.length - 1;
			}
			closeAllSwipesFromStartingPoint(level);
			// delay the removal so the animation can be executed
			$timeout(function() {
				while ($rootScope.swipes.length > level) {
					$rootScope.swipes.pop();
				}
				if ($rootScope.swipes.length === 0 && element.hasClass("open")) {
					element.removeClass("open");
				}
				resizeSwipes();
				deferred.resolve();
			}, 300);
		} else {
			deferred.reject();
		}
	});

	function closeAllSwipesFromStartingPoint(startingPoint) {
		for (var i = $rootScope.swipes.length; i > startingPoint; i--) {
			var swipe = $rootScope.swipes[i - 1];
			swipe.open = false;
			if (swipe.onCloseSwipe) {
				swipe.isMasterDetailPopup = false;
				swipe.onCloseSwipe();
			}
			swipe.restoreLastActiveWidget();
		}
	}

	$scope.$watch("templateManager.container", function() {
		// closes the swipes: 0 = all
		ScreenService.closeSwipe(0);
		templateManager.updateTemplate();
	});

	Zepto($document[0]).swipeRight(function(evt) {
		// prevent closing the swipe if scrolling a zh-tab-wrapper
		var zhTabWrapper = $(evt.target).closest(".zh-tab-wrapper")[0];
		if (zhTabWrapper && zhTabWrapper.scrollWidth > zhTabWrapper.offsetWidth) {
			return;
		}
		// prevent closing the swipe if scrolling a zh-scroller with horizontal scrolling enabled
		var zhScroller = $(evt.target).closest(".container[zh-scroller]")[0];
		if (zhScroller && zhScroller.scrollWidth > zhScroller.offsetWidth) {
			return;
		}
		// prevent closing the swipe if scrolling a grid with horizontal scrolling
		var body = $(evt.target).closest(".body");
		if (body.length > 0) {
			var bodyScrollControl = body.siblings('.body-scroll-control')[0];
			if (bodyScrollControl.scrollWidth > bodyScrollControl.offsetWidth) {
				return;
			}
		}
		if ($scope.widget) {
			$scope.widget.list();
			delete $scope.widget;
		}
		// closes the swipe: no args = topmost
		ScreenService.closeSwipe();
		templateManager.updateTemplate();
	});
}
