describe('DevWindowController', function(){
	
	/**
	 * Dependencies
	 **/
	var templateManager;
	var $scope;

	beforeEach(inject(function($window, $rootScope){
		$scope = $rootScope.$new();
		templateManager = new TemplateManager($window, new ZHPromise(Q));
	}));

	/**
	 * Main Test Suit
	 **/
	describe('Main Test Suit', function(){

		it('$scope.location: should be updated with current URL, but with no hash', function(){
			var fakeWindow = { location: { href: 'http://test-url.com/#/endpoint', hash: '#' }, devtools: false };
			var devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect($scope.location).toBe('http://test-url.com//endpoint');
		});

		it('inDevToolsMode: should returns true if the current URL has the ?devtools query string parameter', function(){
			var fakeWindow = { location: { href: 'http://test-url.com/#/endpoint?devtools=true' } };
			var devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.inDevToolsMode()).toBeTruthy();
		});
		
		it('inDevToolsMode: should return false if URL does not have ?devtools query string parameter', function(){
			var fakeWindow = { location: { href: 'http://test-url.com/#/endpoint' } };
			var devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.inDevToolsMode()).not.toBeTruthy();
		});

		it('isDevToolsOpened: should returns true if $window has devtools property and showDevtoolsInSameWindow as true, or false otherwise', function(){
			var fakeWindow = { location: { href: 'http://test-url.com/#/endpoint?devtools' }};
			fakeWindow.devtools = true;
			fakeWindow.showDevtoolsInSameWindow = true;
			var devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.isDevToolsOpened()).toBeTruthy();

			fakeWindow.devtools = undefined;
			devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.isDevToolsOpened()).not.toBeTruthy();

			fakeWindow.devtools = true;
			fakeWindow.showDevtoolsInSameWindow = false;
			devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.isDevToolsOpened()).not.toBeTruthy();
		});

		it('shouldShowDevtoolsContent: should returns true if is not currently on devToolsMode', function(){
			templateManager.project.inDeveloperMode = true;
			var fakeWindow = { location: { href: 'http://test-url.com/#/endpoint' } };
			var devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.shouldShowDevtoolsContent()).toBeTruthy();

			fakeWindow.location.href = 'http://test-url.com/#/endpoint?devtools=true';
			devWindowController = new DevWindowController(templateManager, fakeWindow, $scope);
			expect(devWindowController.shouldShowDevtoolsContent()).not.toBeTruthy();
		});

	});

});