describe("SynchronizableStorageEngine", function() {

	var primaryStrategy;
	var secondaryStrategy;
	var synchronizableStorageEngineFactory;
	var synchronizableStorageEngine;
	var zhPromise;

	var dataToReturn = ZhHelper.generateData(20);
	var createController;
	var controller;
	var $rootScope;
	var $httpBackend;
	var injector;

	var buildMock = function() {
		var mock = {
			whenSuccess: function(){},
			whenError: function() {}
		};
		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		return mock;
	};

	var fallbackStrategy = {
		onGetDataSourcePrimaryFail: function() {
		},
		onGetDataSourceSecondaryFail: function() {},
		onGetDataSourceBothFail: function() {},
		onSetDataSourcePrimaryFail: function() {},
		onSetDataSourceSecondaryFail: function() {},
		onSetDataSourceBothFail: function() {},
		onSavePrimaryFail: function() {},
		onSaveSecondaryFail: function() {},
		onSaveBothFail: function() {},
		onRemovePrimaryFail: function() {},
		onRemoveSecondaryFail: function() {},
		onRemoveBothFail: function() {},
		onSyncPrimaryFail: function() {},
		onSyncSecondaryFail: function() {},
		onSyncBothFail: function() {}
	};

	Configuration(function(requestDataSourceEngine, ContextRegister, ZhLogger, templateManager, dataSourceHolder, ZHPromise) {
		zhPromise = ZHPromise;
		templateManager.currentWidget = {
			onSync: function() {}
		};
		dataSourceHolder._getDataSource = dataSourceHolder.getDataSource;
		dataSourceHolder.getDataSource = function(name) {
			return dataSourceHolder._getDataSource(name).then(function(data) {
				ZhHelper.securityApply($rootScope);
				return data;
			});
		};
		primaryStrategy = requestDataSourceEngine;
		secondaryStrategy = angular.copy(requestDataSourceEngine);
		ZhLogger.error = function() {}; //ShutUp idiot erros
	});

	beforeEach(function() {
		window.templateUrl = "../../frontend/templates/dyman/";
		window.metadataUrl = "/json_test/test/";
		window.metadataUrls = [];
		window.serviceUrl = "../backend/service/index.php";
		window.serviceUrls = [];
		module('ZeedhiApp');
	});

	beforeEach(inject(function($injector) {
		injector = $injector;
		$httpBackend = $injector.get('$httpBackend');

		$httpBackend.when('GET', '/json_test/test/json/acl.json').respond({}, {});
		$httpBackend.when('GET', '/json_test/test/json/words.json').respond({}, {});
		$httpBackend.when('GET', '/json_test/test/json/words/pt_br.json').respond({}, {});
		$httpBackend.when('GET', '/json_test/test/json/data.json').respond({}, {});
		$httpBackend.when('GET', "/json_test/test/json/datasources/countries.json").respond({
			countries: {
				localKey: ['id']
			}
		}, {});
		$httpBackend.when('GET', "/json_test/test/json/containers.json").respond(ZhHelper.projectTest(), {});
		$httpBackend.when('GET', "/json_test/test/json/environment.json").respond(ZhHelper.emptyEnvironment, {});
		$httpBackend.when('GET', "/json_test/test/json/containers/login.json").respond(ZhHelper.emptyContainer, {});
		$httpBackend.when('GET', "/json_test/test/json/datasources/zhPreferences.json").respond(200, {});

		/* bower.json request (because of templateManager.updateVersionData()) */
		$httpBackend.when('GET', './bower.json').respond({}, {});

		$rootScope = $injector.get('$rootScope');

		var $controller = $injector.get('$controller');
		createController = function(controllerName, scope) {
			return $controller(controllerName, {
				'$scope': scope
			});
		};

		synchronizableStorageEngineFactory = $injector.get('SynchronizableStorageEngineFactory');
		synchronizableStorageEngine = synchronizableStorageEngineFactory.factory(primaryStrategy, secondaryStrategy, fallbackStrategy, false, false);

		$rootScope.$apply();
	}));

	afterEach(function() {
		$httpBackend.verifyNoOutstandingExpectation();
		$httpBackend.verifyNoOutstandingRequest();
	});

	it('SynchronizableStorageEngine must be instantiated correctly', function() {
		expect(synchronizableStorageEngine).toEqual(jasmine.any(SynchronizableStorageEngine));
		//expect(synchronizableStorageEngine.primaryStrategy).toEqual(jasmine.any(RequestDataSourceEngine));
		//expect(synchronizableStorageEngine.secondaryStrategy).toEqual(primaryStrategy);
		//expect(synchronizableStorageEngine.fallbackStrategy).toEqual(fallbackStrategy);
		//expect(synchronizableStorageEngine.alwaysSync).toEqual(false);
		//expect(synchronizableStorageEngine.secondaryRequired).toEqual(false);
	});

	it('Method "getDataSource": The primaryStrategy must return the data if success', function() {
		$httpBackend.expectPOST("../backend/service/index.php/countries").respond(200, ZhHelper.repositoryRequestMock("countries", dataToReturn));
		var done = null;
		synchronizableStorageEngine.getDataSource('/countries').then(function(data) {
			expect(data.dataset.countries.length).toEqual(21);
			done();
		}, function(err) {
			expect('Error').toEqual(false);
			done();
		});
		ZhHelper.securityApply($rootScope);
		ZhHelper.doAsyncTest(function(_done) {
			done = _done;
			ZhHelper.securityApply($rootScope);
		}, function() {}, 1000, $rootScope, $httpBackend);
	});

	it('Method "getDataSource": The secondaryStrategy must return the data if an error occurred with the primaryStrategy', function(done) {
		controller = createController('ZeedhiMain', $rootScope);

		$httpBackend.expectPOST("../backend/service/index.php/countries").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries").respond(200, ZhHelper.repositoryRequestMock("countries", dataToReturn.concat({})));

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourceBothFail').and.callThrough();

		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.getDataSource('/countries', function(){
				ZhHelper.securityApply($rootScope);
			}, undefined, undefined, undefined, function(){
				ZhHelper.securityApply($rootScope);
			}).then(function(data){
				expect(data.dataset.countries.length).toEqual(22);
				mock.whenSuccess();
			}, mock.whenError);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, true);
	});

	it('Method "getDataSource": If both storage strategies has fail, the 3 fallback methods must be called', function(done) {
		controller = createController('ZeedhiMain', $rootScope);
		$httpBackend.expectPOST("../backend/service/index.php/countries").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries").respond(500, {});

		var mock = buildMock();

		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onGetDataSourceBothFail').and.callThrough();

		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.getDataSource('/countries', function() {
				ZhHelper.securityApply($rootScope);
			}, undefined, undefined, undefined, function() {
				ZhHelper.securityApply($rootScope);
			}).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			expect(mock.whenError).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourceSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onGetDataSourceBothFail).toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, true);
	});

	it('Method "setDataSource": If the primaryStrategy has success, the success promise must be called and no fallback must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the primaryStrategy has fail, the fallback method must be called and the secondaryStrategy too. If the secondaryStrategy has success, the success promise must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If both storage strategies has fail, the 3 fallback methods must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			expect(mock.whenError).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the secondaryStrategy is required, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the secondaryStrategy is required, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the secondaryStrategy is required, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the alwaysSync is true, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the alwaysSync is true, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "setDataSource": If the alwaysSync is true, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'setDataSource').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourcePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSetDataSourceBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.setDataSource('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.setDataSource).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourcePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSetDataSourceBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the primaryStrategy has success, the success promise must be called and no fallback must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, true);
	});

	it('Method "save": If the primaryStrategy has fail, the fallback method must be called and the secondaryStrategy too. If the secondaryStrategy has success, the success promise must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If both storage strategies has fail, the 3 fallback methods must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			expect(mock.whenError).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the secondaryStrategy is required, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the secondaryStrategy is required, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the secondaryStrategy is required, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the alwaysSync is true, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the alwaysSync is true, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "save": If the alwaysSync is true, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/save").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'save').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSavePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSaveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.save('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.save).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSavePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSaveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the primaryStrategy has success, the success promise must be called and no fallback must be called', function(done) {
		controller = createController('ZeedhiMain', $rootScope);
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(function() {
				ZhHelper.securityApply($rootScope);
				mock.whenSuccess();
			}, mock.whenError);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, true);
	});

	it('Method "remove": If the primaryStrategy has fail, the fallback method must be called and the secondaryStrategy too. If the secondaryStrategy has success, the success promise must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If both storage strategies has fail, the 3 fallback methods must be called', function(done) {
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			expect(mock.whenError).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the secondaryStrategy is required, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the secondaryStrategy is required, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the secondaryStrategy is required, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the alwaysSync is true, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the alwaysSync is true, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "remove": If the alwaysSync is true, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(200, {});
		$httpBackend.expectPOST("../backend/service/index.php/countries/remove").respond(500, {});

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'remove').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemovePrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onRemoveBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.remove('/countries', ZhHelper.generateData(20)).then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function() {
			ZhHelper.securityApply($rootScope);
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.remove).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemovePrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onRemoveBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend);
	});

	it('Method "sync": If the primaryStrategy has success, the success promise must be called and no fallback must be called', function(done) {
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();

		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the primaryStrategy has fail, the fallback method must be called and the secondaryStrategy too. If the secondaryStrategy has success, the success promise must be called', function(done) {
		synchronizableStorageEngine.primaryStrategy.sync = function() {
			return zhPromise.reject();
		};

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If both storage strategies has fail, the 3 fallback methods must be called', function(done) {
		synchronizableStorageEngine.primaryStrategy.sync = synchronizableStorageEngine.secondaryStrategy.sync = function() {
			return zhPromise.reject();
		};
		var mock = buildMock();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).not.toHaveBeenCalled();
			expect(mock.whenError).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the secondaryStrategy is required, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the secondaryStrategy is required, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;
		synchronizableStorageEngine.primaryStrategy.sync = function() {
			return zhPromise.reject();
		};

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the secondaryStrategy is required, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.secondaryRequired = true;
		synchronizableStorageEngine.secondaryStrategy.sync = function() {
			return zhPromise.reject();
		};

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the alwaysSync is true, he must be called if the first storeStrategy has success', function(done) {
		synchronizableStorageEngine.alwaysSync = true;

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the alwaysSync is true, he must be called if the first storeStrategy has fail', function(done) {
		synchronizableStorageEngine.alwaysSync = true;
		synchronizableStorageEngine.primaryStrategy.sync = function() {
			return zhPromise.reject();
		};

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

	it('Method "sync": If the alwaysSync is true, and has fail, the fallback method must be called', function(done) {
		synchronizableStorageEngine.alwaysSync = true;
		synchronizableStorageEngine.secondaryStrategy.sync = function() {
			return zhPromise.reject();
		};

		var mock = buildMock();
		spyOn(synchronizableStorageEngine.primaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.secondaryStrategy, 'sync').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncPrimaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncSecondaryFail').and.callThrough();
		spyOn(synchronizableStorageEngine.fallbackStrategy, 'onSyncBothFail').and.callThrough();
		ZhHelper.doAsyncTest(function(_done) {
			synchronizableStorageEngine.sync('/countries').then(mock.whenSuccess, mock.whenError);
		}, function() {
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.primaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.secondaryStrategy.sync).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncPrimaryFail).not.toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncSecondaryFail).toHaveBeenCalled();
			expect(synchronizableStorageEngine.fallbackStrategy.onSyncBothFail).not.toHaveBeenCalled();
			done();
		}, 1000, $rootScope, $httpBackend, false);
	});

});
