describe("ZhFeatures", function () {
	var zhPromise = new ZHPromise(Q);
	it("Request as PostData wil be removed on 31/12/2014", function () {
		var project = new Project({}, {});
		var now = new Date();
		var expirationDatePostData = new Date('Wed Dec 31 2014 12:00:32 GMT-0200 (BRST)');
		if (expirationDatePostData.getTime() > now.getTime()) {
			expect(project.usePostData).toBe(true);
		} else {
			expect(project.usePostData).toBe(false);
		}
	});
	it("Request as PostData wil be removed on 31/12/2014 - feature test", function () {
		var lastParams = {};
		var zhFeatures = new ZhFeatures(console);
		var project = new Project({}, {});
		zhFeatures.usePostData = project.usePostData;
		var templateManager = {
			getMappedPackages: function () {
				return {
					default: ["./"]
				};
			},
			getMappedServicePackages: function () {
				return {
					default: ["./"]
				};
			},
			serviceUrl: "",
			serviceUrls: ["a"],
			projectPromise: zhPromise.when(),
			project: ZhHelper.projectTest().zeedhi_project
		};
		var requestInterceptingEngineMock = {
			intercept: function () {
				return {
					then: function (fn)
					{
						if (fn) {
							fn();
						}
					}
				};
			}
		};
		var rest = new RestEngine(console, function (p) {
			lastParams = p;
			return {
				success: function () {
					return {
						then: function () {

						}
					};
				}
			};
		}, templateManager, {}, zhFeatures, zhPromise, {	onRequestStart: function () { }}, requestInterceptingEngineMock);
		var testObj = {
			"teste": "test"
		};
		rest.request("", testObj, function () { }, function () { }, function () { });
		var now = new Date();
		var expirationDatePostData = new Date('Wed Dec 31 2014 12:00:32 GMT-0200 (BRST)');
		if (expirationDatePostData.getTime() > now.getTime()) {
			expect(lastParams.data).toBe("teste=test");
		} else {
			expect(lastParams.data).toBe(testObj);
		}

	});

	it("localStorageEngine will be removed on 2015-04-13", function () {
		var now = new Date();
		var expirationDate = new Date('Fri Apr 17 2015 09:00:00 GMT-0300 (Hora oficial do Brasil)');
		if (expirationDate.getTime() < now.getTime()) {
			expect(!window.localStorageEngine).toBe(true);
		}
	});

	it("Should send warning if _usePostData have been used!", function () {
		spyOn(console, 'warn').and.callThrough();

		var zhf = new ZhFeatures();
		zhf.usePostData = true;
		if (zhf.usePostData) { } //Nessesary for the test.

		expect(console.warn).toHaveBeenCalled();
	});

	describe("ZhFeatureUsage", function () {
		var backupWebSocket = window.WebSocket;
		var ApplicationContext = {};
		var templateManager = {
			project: {
				name: "tst",
				zhFeaturesWebSocketUrl: "ws://ws.zeedhi.com:8080",
				zhFeaturesWebSocketRoom: "spec_test_zeedhi_features_usage",
				zhFeatureUsageEnabled: true
			},
			container: {
				name: "tstContainer",
				updateCheckedRows: function () { }
			},
			updateTemplate: function () { }
		};
		ApplicationContext.templateManager = templateManager;
		var zhPromise = new ZHPromise(Q);
		ApplicationContext.ZHPromise = zhPromise;
		var zhInnerSocket;
		var messagesSentToWebSocket = [];

		var mockWebSocket = function (host) {
			var self = this;
			this.host = host;
			zhPromise.when().then(function () {
				self.onopen();
			});
			this.send = function (message) {
				messagesSentToWebSocket.push(message);
				self.onmessage(message);
			};
		};

		var zhSocketFactory = {
			create: function (socketClient, host) {
				return zhInnerSocket.create(socketClient, host, templateManager);
			}
		};
		ApplicationContext.ZhSocketFactory = zhSocketFactory;

		zhFeatures = new ZhFeatures(ApplicationContext);

		var container;
		var field;


		var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = null;
		/*jslint evil: true */
		var eventEngine = new EventEngine(new EventFactory(new MethodBuilder({}, {
			$eval: function (a, b) {
				return eval(a, b);
			}
		}), {}), new EventHandler({}, {}, console), {
				updateTemplate: function () { }
			}, {
				i18n: function (text) {
					return text;
				}
			});
		var aclEngine = new AclEngine();
		var screenService = {
			isPopupOpen: function () {
				return false;
			}
		};
		var metaDataFactory;

		var baseContainer = function () {
			return {
				widgets: [{
					name: "empty",
					fields: [{
						name: "testField",
						isVisible: true,
						dataSource: {},
						template: "field/select.html"
					}],
					dataSources: {},
					events: []
				}]
			};
		};

		beforeAll(function () {
			window.WebSocket = mockWebSocket;
		});

		afterAll(function () {
			window.WebSocket = backupWebSocket;
		});

		beforeEach(function () {
			zhFeatures = new ZhFeatures(ApplicationContext);
			metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
				SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, templateManager,
				{}, new ZHPromise(Q), { getDataSourceLazy: function () { } }, {}, {}, zhFeatures);

			messagesSentToWebSocket = [];
			zhInnerSocket = new ZhInnerSocket({ error: function () { } });

			container = metaDataFactory.containerFactory(baseContainer());
			field = container.widgets[0].fields[0];
		});

		function testFieldFeature(featureName, done) {
			field[featureName]();

			setTimeout(function () {
				var parsedMessages = messagesSentToWebSocket.map(function (message) { return JSON.parse(message); });
				expect(parsedMessages.length).toBe(3);

				var message = parsedMessages.shift();
				expect(message).toEqual({
					namespace: "zh_features",
					room: "spec_test_zeedhi_features_usage",
					event: "createNamespace",
					data: {
						event: "createNamespace",
						key: "you can create!"
					}
				});

				message = parsedMessages.shift();
				expect(message).toEqual({
					namespace: "zh_features",
					room: "spec_test_zeedhi_features_usage",
					event: "createRoom",
					data: {
						event: "createRoom",
						key: "you can create!"
					}
				});

				message = parsedMessages.shift();
				expect(message.namespace).toBe("zh_features");
				expect(message.room).toBe("spec_test_zeedhi_features_usage");
				expect(message.data.caller).toBe(featureName);
				expect(message.data.containerName).toBe("tstContainer");
				expect(message.data.fieldName).toBe("testField");
				expect(message.data.key).toBe("you can create!");
				expect(message.data.projectName).toBe("tst");
				expect(message.data.userId).toEqual(localStorage.getItem("USER_ID"));
				expect(message.data.timestamp).toBeDefined();

				done();
			}, 50);
		}

		it("Field.getDataSource: should send a message to the ZhFeatures websocket", function (done) {
			testFieldFeature("getDataSource", done);
		});

		it("Field.show: should send a message to the ZhFeatures websocket", function (done) {
			testFieldFeature("show", done);
		});

		it("Field.hide: should send a message to the ZhFeatures websocket", function (done) {
			testFieldFeature("hide", done);
		});

		it("ZhFeatures.registerFeatureUsage: should do nothing if zhFeatureUsage is disabled in the project", function () {
			templateManager.container.zhFeatureUsageEnabled = false;

			zhFeatures.registerFeatureUsage();
			expect(messagesSentToWebSocket.length).toBe(0);
		});
	});
});