#!/bin/bash

# Set the Apache server address and folder path
SERVER_ADDRESS="<>SERVER_IP_PORT<>"
FOLDER_PATH="/updates/"

# Set the local installation directory
POOL_DIR="/usr/local/Teknisa/"
KDS_INSTALL_DIR="/usr/local/kds/"

if [ ! -d "$POOL_DIR" ]; then
    sudo mkdir -p "$POOL_DIR"
    sudo chmod 777 "$POOL_DIR"
fi

# Declare an associative array
declare -A UPDATE_INSTALLED


# Loop over each file in the folder on the server
for FILE in $(curl -s "$SERVER_ADDRESS$FOLDER_PATH" | grep -Eo 'href="[^"]+"'); do
    # Extract the filename from the URL
    FILENAME=$(echo "$FILE" | sed 's/href="//; s/\/$//; s/"$//')

    # Extract the app name and version from the filename
    APP_NAME=$(echo "$FILENAME" | sed 's/[0-9]\+\.zip$//')
    VERSION=$(echo "$FILENAME" | sed 's/^[^0-9]*\([0-9]\+\).*\.zip$/\1/')

    if [ "$FILENAME" != "/" ]; then

        # Check if the file is already installed
        INSTALLED_FILE=($POOL_DIR$APP_NAME*)

        if [ -e "${INSTALLED_FILE[0]}" ]; then

            # Extract the version number from the installed file name
            INSTALLED_VERSION=$(echo "${INSTALLED_FILE[0]}" | sed 's/^[^0-9]*\([0-9]\+\).*\.zip$/\1/')

            # Compare the version numbers
            if [[ "$APP_NAME" == "kds" && "$VERSION" > "$INSTALLED_VERSION" ]]; then
                
                # Remove the existing installation
                sudo rm -rf "$KDS_INSTALL_DIR"
                sudo rm -f "$POOL_DIR$APP_NAME"*

                # Download and install the newer version
                echo "Downloading $FILENAME..."
                sudo curl -s -o "$POOL_DIR$FILENAME" "$SERVER_ADDRESS$FOLDER_PATH$FILENAME"
                sudo unzip -q "$POOL_DIR$FILENAME" -d "$KDS_INSTALL_DIR"
                chmod +x "$KDS_INSTALL_DIR"KDS
                echo "Installed $FILENAME."
                UPDATE_INSTALLED["$APP_NAME"]="$APP_NAME$VERSION"
            fi
        else            
            # File is not installed, download and install it
            sudo rm -f "$POOL_DIR$APP_NAME"*

            echo "Downloading $FILENAME..."
            sudo curl -s -o "$POOL_DIR$FILENAME" "$SERVER_ADDRESS$FOLDER_PATH$FILENAME"
            sudo rm -rf "$KDS_INSTALL_DIR"
            sudo unzip -q "$POOL_DIR$FILENAME" -d "$KDS_INSTALL_DIR"
            chmod +x "$KDS_INSTALL_DIR"KDS
            echo "Installed $FILENAME."
            UPDATE_INSTALLED["$APP_NAME"]="$APP_NAME$VERSION"
        fi
    fi
done

for key in "${!UPDATE_INSTALLED[@]}"; do
    echo "$key: ${UPDATE_INSTALLED[$key]}"
done

echo "Finished updating Teknisa Front Apps."
